package com.company.chartsaddon.view.nativejson;

import com.company.chartsaddon.view.main.MainView;
import com.storedobject.chart.*;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.router.Route;
import io.jmix.chartsflowui.component.Chart;
import io.jmix.chartsflowui.kit.component.ChartRenderer;
import io.jmix.chartsflowui.kit.component.event.ChartClickEvent;
import io.jmix.flowui.Notifications;
import io.jmix.flowui.kit.component.button.JmixButton;
import io.jmix.flowui.view.*;
import org.springframework.beans.factory.annotation.Autowired;

@Route(value = "nativejson", layout = MainView.class)
@ViewController("NativeJsonView")
@ViewDescriptor("native-json-view.xml")
public class NativeJsonView extends StandardView {

    @ViewComponent
    private Chart chart;
    @Autowired
    private Notifications notifications;

    @Subscribe
    public void onInit(InitEvent event) {
        SOChart soChart = new SOChart();

        soChart.setWidth("99%");
        soChart.setMinHeight("30em");

        CategoryData labels = new CategoryData("Backlog", "Next", "In-progress", "Review", "QA", "Done");
        Data data = new Data(18, 8, 5, 6, 29, 53);

        BarChart barChart = new BarChart(labels, data);
        RectangularCoordinate coordinate =
                new RectangularCoordinate(new XAxis(DataType.CATEGORY), new YAxis(DataType.NUMBER));
        barChart.plotOn(coordinate);

        barChart.setName("Bars");

        Toolbox toolbox = new Toolbox();
        toolbox.addButton(new Toolbox.Zoom(), new Toolbox.Download());

        Title title = new Title("Issues by status");
        title.setSubtext("The current status is presented in bars");

        soChart.add(barChart, toolbox, title);

        getContent().add(soChart);
    }

    @Subscribe("chart")
    public void onChartChartClick(final ChartClickEvent event) {
        notifications.show("test");
    }

    @Subscribe(id = "test", subject = "clickListener")
    public void onTestClick(final ClickEvent<JmixButton> event) {
        chart.getElement().executeJs("""
                    var svgBlob = new Blob([$0._root.renderToSVGString()], {type:"image/svg+xml;charset=utf-8"});
                    var svgUrl = URL.createObjectURL(svgBlob);
                    var downloadLink = document.createElement("a");

                    downloadLink.href = svgUrl;
                    downloadLink.download = "newesttree.svg";
                    document.body.appendChild(downloadLink);
                    downloadLink.click();
                    document.body.removeChild(downloadLink);
                """);
    }
}
