package com.company.chartsaddon.view.user;

import com.company.chartsaddon.entity.User;
import com.company.chartsaddon.security.FullAccessRole;
import com.company.chartsaddon.view.main.MainView;
import com.vaadin.flow.router.Route;
import io.jmix.chartsflowui.component.Chart;
import io.jmix.chartsflowui.data.item.MapDataItem;
import io.jmix.chartsflowui.kit.component.model.DataSet;
import io.jmix.chartsflowui.kit.data.chart.ListChartItems;
import io.jmix.core.DataManager;
import io.jmix.core.querycondition.LogicalCondition;
import io.jmix.core.querycondition.PropertyCondition;
import io.jmix.flowui.component.propertyfilter.PropertyFilter;
import io.jmix.flowui.model.CollectionContainer;
import io.jmix.flowui.view.*;
import io.jmix.securitydata.entity.RoleAssignmentEntity;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Map;

@Route(value = "users", layout = MainView.class)
@ViewController("User.list")
@ViewDescriptor("user-list-view.xml")
@LookupComponent("usersDataGrid")
@DialogMode(width = "64em")
public class UserListView extends StandardListView<User> {

    @ViewComponent
    private Chart chart;

    @ViewComponent
    private CollectionContainer<User> usersDc;
    @Autowired
    private DataManager dataManager;

    @Subscribe
    protected void onInit(ReadyEvent event) {
        chart.setDataSet(createDateSet());
    }

    protected DataSet createDateSet() {
        DataSet dataSet = new DataSet();

        DataSet.Source<MapDataItem> itemSource = new DataSet.Source<>();

        List<String> admins = dataManager.load(RoleAssignmentEntity.class)
                .condition(
                        new LogicalCondition(LogicalCondition.Type.AND)
                                .add(PropertyCondition.equal("roleCode", FullAccessRole.CODE))
                )
                .list()
                .stream()
                .map(RoleAssignmentEntity::getUsername)
                .toList();

        itemSource.withDataProvider(
                        new ListChartItems<>(
                                new MapDataItem(Map.of("Role: ", "Admins", "count", getAdminCount(admins))),
                                new MapDataItem(Map.of("Role: ", "Users", "count", getUsersCount(admins)))
                        )
                )
                .withCategoryField("Role: ")
                .withValueField("count");

        dataSet.setSource(itemSource);

        return dataSet;
    }

    protected Long getAdminCount(List<String> admins) {
        return usersDc.getItems().stream()
                .filter(user -> admins.contains(user.getUsername()))
                .count();
    }

    protected Long getUsersCount(List<String> admins) {
        return usersDc.getItems().stream()
                .filter(user -> !admins.contains(user.getUsername()))
                .count();
    }
}